/*! \file
    \brief Debugging message file

    This file provides mappings from various code numbers to strings.\n
    It is only used for debugging messages.
*/
#include <stdlib.h>
#include "CmdTable.h"

//! Command String from Command #
static const char *const commandTable[]= {
NULL,                    // 0
NULL,                    // 1
NULL,                    // 2
NULL,                    // 3
NULL,                    // 4
NULL,                    // 5
NULL,                    // 6
NULL,                    // 7
"CMD_DEBUG",             // 8  Debugging commands (parameter determines actual command)
NULL,                    // 9
"CMD_OPTION",            // 10 Get/set various options
NULL,                    // 11
"CMD_GET_VER",           // 12 returns 16 bit HW/SW version number, (major & minor revision
"CMD_GET_LAST_STATUS",   // 13 returns status of the previous command
"CMD_SET_BOOT",          // 14 request boot-loader firmware upgrade on next power-up, parameters: 'B','O','O','T', returns: none
NULL,                    // 15
NULL,                    // 16
NULL,                    // 17
NULL,                    // 18
NULL,                    // 19
NULL,                    // 20
NULL,                    // 21
NULL,                    // 22
NULL,                    // 23
NULL,                    // 24
NULL,                    // 25
NULL,                    // 26
NULL,                    // 27
NULL,                    // 28
NULL,                    // 29
"CMD_SET_TARGET",        // 30 set target, 8bit parameter: 00=HC12/HCS12(default), 01=HCS08
"CMD_CONNECT",           // 31 try to connect to the target
NULL,                    // 32
"CMD_RESET",             // 33 8-bit parameter: 0=reset to Special Mode, 1=reset to Normal mode
"CMD_GET_STATUS",        // 34 returns 16bit status word:
"CMD_READ_BD",           // 35 parameter: 16-bit address, returns 8-bit value read from address;
"CMD_WRITE_BD",          // 36 parameter: 16-bit address, 8-bit value to write;
NULL,                    // 37
NULL,                    // 38
"CMD_HALT",              // 39 stop the CPU and bring it into background mode
NULL,                    // 40 deprecated
"CMD_READ_SPEED",        // 41 read speed of the target: returns 16-bit tick count
"CMD_GO",                // 42 start code execution
NULL,                    // 43
"CMD_STEP",              // 44 perform single step
NULL,                    // 45
"CMD_SET_SPEED1",        // 46 sets-up the BDM interface for a new bit rate & tries
"CMD_SET_DERIVATIVE",    // 47
NULL,                    // 48
NULL,                    // 49
"CMD_READ_8",            // 50 parameter 16bit address, returns 8bit value read from address
"CMD_READ_16",           // 51 parameter 16bit address, returns 16bit value read from address
NULL,                    // 52
"CMD_READ_REGS",         // 53 reads registers, returns 16bit values:
NULL,                    // 54
"CMD_READ_BLOCK1",       // 55 parameter 16bit address, 8bit count of bytes to read,
NULL,                    // 56
NULL,                    // 57
NULL,                    // 58
NULL,                    // 59
"CMD_WRITE_8",           // 60 parameter 16bit address, 8bit value to write
"CMD_WRITE_16",          // 61 parameter 16bit address, 16bit value to write
NULL,                    // 62
NULL,                    // 63
NULL,                    // 64
"CMD_WRITE_BLOCK1",      // 65 parameters: 16bit address, 8bit count of bytes,
NULL,                    // 66
NULL,                    // 67
NULL,                    // 68
NULL,                    // 69
NULL,                    // 70
NULL,                    // 71
NULL,                    // 72
NULL,                    // 73
NULL,                    // 74
NULL,                    // 75
NULL,                    // 76
NULL,                    // 77
NULL,                    // 78
NULL,                    // 79
"CMD_WRITE_REG_PC",      // 80 parameter: 16-bit PC value
"CMD_WRITE_REG_SP",      // 81 parameter: 16-bit SP value
"CMD_WRITE_REG_X",       // 82 parameter: 16-bit IX (H:X) value
"CMD_WRITE_REG_Y",       // 83 parameter: 16-bit IY value
"CMD_WRITE_REG_D",       // 84 parameter: 16-bit B:A (x:A) value
"CMD_WRITE_REG_CCR",     // 85 parameter: 16-bit CCR (x:CCR) value
"CMD_CF_READ_CSR2",      // 86
"CMD_CF_READ_CSR3",      // 87
"CMD_CF_WRITE_CSR2",     // 88
"CMD_CF_WRITE_CSR3",     // 89
"CMD_CF_READ_REG",       // 90
"CMD_CF_READ_CREG",      // 91
"CMD_CF_READ_DREG",      // 92
"CMD_CF_READ_XCSR",      // 93
"CMD_CF_WRITE_REG",      // 94
"CMD_CF_WRITE_CREG",     // 95
"CMD_CF_WRITE_DREG",     // 96
"CMD_CF_READ_MEM",       // 97
"CMD_CF_WRITE_MEM",      // 98
"CMD_CF_WRITE_XCSR",     // 99
"CMD_READ_STATUS",       // 100
"CMD_WRITE_CONTROL",     // 101
"CMD_WRITE_BKPT",        // 102
"CMD_READ_BKPT",         // 103
"CMD_READ_SPC",          // 104
"CMD_WRITE_SPC",         // 105
"CMD_READ_CCR_PC",       // 106
"CMD_WRITE_CCR_PC",      // 107
NULL,                    // 108
NULL,                    // 109
"CMD_MASS_ERASE",        // 110
NULL,                    // 111
NULL,                    // 112
"CMD_FLASH_DLSTART",     // 113
"CMD_FLASH_DLEND",       // 114
"TCMD_NEXT_DLDATA",      // 115
NULL,                    // 116
NULL,                    // 117
NULL,                    // 118
NULL,                    // 119
"CMD_CFVx_RESYNCHRONIZE",        // 120  Resynchronize communication with the target (in case of noise, etc.) */
"CMD_CFVx_RESET",                // 121, Reset target @param [2] 0=>reset to Special Mode, 1=>reset to Normal mode
"CMD_CFVx_GO",                   // 122, Start code execution
"CMD_CFVx_STEP",                 // 123, Perform single step
"CMD_CFVx_HALT",                 // 124, Stop the CPU and bring it into background mode
"CMD_CFVx_ASSERT_TA",            // 125  parameter: 8-bit number of 10us ticks - duration of the TA assertion */
NULL,                            // 126
NULL,                            // 127
NULL,                            // 128
NULL,                            // 129
"CMD_CFVx_READ_MEM8",            // 130, parameter 32bit address, returns 8bit value read from address */
"CMD_CFVx_READ_MEM16",           // 131, parameter 32bit address, returns 16bit value read from address */
"CMD_CFVx_READ_MEM32",           // 132, parameter 32bit address, returns 32bit value read from address */
"CMD_CFVx_READ_MEMBLOCK8",       // 133, parameter 32bit address, returns block of 8bit values read from the address and onwards */
"CMD_CFVx_READ_MEMBLOCK16",      // 134, parameter 32bit address, returns block of 16bit values read from the address and onwards */
"CMD_CFVx_READ_MEMBLOCK32",      // 135, parameter 32bit address, returns block of 32bit values read from the address and onwards */
"CMD_CFVx_WRITE_MEM8",           // 136, parameter 32bit address & an 8-bit value to be written to the address */
"CMD_CFVx_WRITE_MEM16",          // 137, parameter 32bit address & a 16-bit value to be written to the address */
"CMD_CFVx_WRITE_MEM32",          // 138, parameter 32bit address & a 32-bit value to be written to the address */
"CMD_CFVx_CFVx_WRITE_MEMBLOCK8", // 139, parameter 32bit address & a block of 8-bit values to be written from the address */
"CMD_CFVx_WRITE_MEMBLOCK16",     // 140, parameter 32bit address & a block of 16-bit values to be written from the address */
"CMD_CFVx_WRITE_MEMBLOCK32",     // 141, parameter 32bit address & a block of 32-bit values to be written from the address */
"CMD_CFVx_READ_REG",             // 142, parameter 8-bit register number to read, returns 32-bit register contents */
"CMD_CFVx_CFVx_WRITE_REG",       // 143, parameter 8-bit register number to write & the 32-bit register contents to be written */
"CMD_CFVx_READ_CREG",            // 144, parameter 16-bit register address, returns 32-bit control register contents */
"CMD_CFVx_WRITE_CREG",           // 145, parameter 16-bit register address & the 32-bit control register contents to be written */
"CMD_CFVx_READ_DREG",            // 146, parameter 8-bit register number to read, returns 32-bit debug module register contents */
"CMD_CFVx_WRITE_DREG",           // 147, parameter 8-bit register number to write & the 32-bit debug module register contents to be written */
NULL,                            // 148
NULL,                            // 149
"CMD_JTAG_GOTORESET",    // 150, no parameters, takes the TAP to TEST-LOGIC-RESET state, re-select the JTAG target to take TAP back to RUN-TEST/IDLE */
"CMD_JTAG_GOTOSHIFT",    // 151, parameters 8-bit path option; path option ==0 : go to SHIFT-DR, !=0 : go to SHIFT-IR (requires the tap to be in RUN-TEST/IDLE) */
"CMD_JTAG_WRITE",        // 152, parameters 8-bit exit option, 8-bit count of bits to shift in, and the data to be shifted in (shifted in LSB (last byte) first, unused bits (if any) are in the MSB (first) byte; exit option ==0 : stay in SHIFT-xx, !=0 : go to RUN-TEST/IDLE when finished */
"CMD_JTAG_READ",         // 153, parameters 8-bit exit option, 8-bit count of bits to shift out; exit option ==0 : stay in SHIFT-xx, !=0 : go to RUN-TEST/IDLE when finished, returns the data read out of the device (first bit in LSB of the last byte in the buffer) */
};

//! \brief Maps a command code to a string
//!
//! @param command Command number
//!
//! @return pointer to static string describing the command
//!
const char *getCommandName(unsigned char command) {
   static char const *commandName = NULL;
   if (command < sizeof(commandTable)/sizeof(commandTable[0]))
      commandName = commandTable[command];
   if (commandName == NULL)
      commandName = "UNKNOWN";
   return commandName;
}

//! Error message string from Error #
static const char *const errorMessages[] = {
    "No Error",                           // 0
    "Illegal parameters",                 // 1
    "General Fail (for compatibility)",   // 2
    "Busy",                               // 3
    "Illegal command",                    // 4
    "No connection to target",            // 5
    "Overrun",                            // 6
    "CF Illegal Command Response",        // 7
    NULL,                                 // 8
    NULL,                                 // 9
    NULL,                                 // 10
    NULL,                                 // 11
    NULL,                                 // 12
    NULL,                                 // 13
    NULL,                                 // 14
    "Unknown Target",                     // 15
    "No Tx routine was found",            // 16
    "No Rx routine was found",            // 17
    "BDM Enable failed",                  // 18
    "Reset pin timeout",                  // 19
    "BKGD pin timeout",                   // 20
    "SYNC timeout",                       // 21
    "Unable to determine speed",          // 22
    "Wrong Flash programming mode",       // 23
    "Busy Flash Programming",             // 24
    "Vdd not removed",                    // 25
    "Vdd not present",                    // 26
    "Vdd not controlled by BDM",          // 27
    "Error terminated bus cycle",         // 28
    "USB Transfer error",                 // 29
};

//! \brief Maps a Error Code # to a string
//!
//! @param error Error number
//!
//! @return pointer to static string describing the error
//!
const char *getErrorName(unsigned char error) {
   char const *errorName = NULL;

   if (error < sizeof(errorMessages)/sizeof(errorMessages[0]))
      errorName = errorMessages[error];
   if (errorName == NULL)
      errorName = "UNKNOWN";
   return errorName;
}

//! Error message string from Error #
static const char *const debugCommands[] = {
   "ACKN",              // 0
   "SYNC",              // 1
   "Test Port",         // 2
   "USB Disconnect",    // 3
   "Find Stack size",   // 4
   "Vpp Off",           // 5
   "Vpp On",            // 6
   "Flash 12V Off",     // 7
   "Flash 12V On",      // 8
   "Vdd Off",           // 9
   "Vdd 3.3V On",       // 10
   "Vdd 5V On",         // 11
   "Cycle Vdd",         // 12
   "Measure Vdd",       // 13
};

//! \brief Maps a Debug Command # to a string
//!
//! @param cmd Debug command number
//!
//! @return pointer to static string describing the command
//!
const char *getDebugCommandName(unsigned char cmd) {
   char const *cmdName = NULL;
   if (cmd < sizeof(debugCommands)/sizeof(debugCommands[0]))
      cmdName = debugCommands[cmd];
   if (cmdName == NULL)
      cmdName = "UNKNOWN";
   return cmdName;
}

//! Option command message string from Option command #
static const char *const optionCommands[] = {
   "Get Capabilities",                             // 0
   "Set Target Vdd (Off,3.3V,5V)",                 // 1
   "Set Cycle Target Vdd on reset",                // 2
   "Set Automatic re-connect",                     // 3
   "Set Guess speed (HC12 only)",                  // 4
   "Set Leave Target Powered on Exit",             // 5
   "Set Use Alternate BDM Target Clock",           // 6
   "Set Cycle Target Vdd on connection problems",  // 7
   "Set Use BDM RESET signal",                     // 8
   "Set Trim frequency (SYNC value)",              // 9
   "Set Interface (BKGD/RESET) High/Low",          // 10
   };

//! \brief Maps a Option Command # to a string
//!
//! @param cmd Option command number
//!
//! @return pointer to static string describing the command
//!
const char *getOptionCommandName(unsigned char cmd) {
   char const *cmdName = NULL;
   if (cmd < sizeof(optionCommands)/sizeof(optionCommands[0]))
      cmdName = optionCommands[cmd];
   if (cmdName == NULL)
      cmdName = "UNKNOWN";
   return cmdName;
}

char const *getCFV2ControlRegName( unsigned int regAddr){
static const char *regs[] = {"D0","D1","D2","D3","D4","D5","D6","D7",
                        "A0","A1","A2","A3","A4","A5","A6","USP"};
static const char *regs2[] = {"SSP","VBR",NULL,NULL,"MACSR","MASK","ACC",NULL,
                         NULL,NULL,NULL,NULL,NULL,NULL,"SR","PC"};
const char *regName = NULL;

   if ((regAddr >= 0x080) && (regAddr <= 0x08F)) // Register D0-A7
      regName = regs[regAddr-0x80];
   else if ((regAddr >= 0x180) && (regAddr <= 0x18F)) // Register D0-A7
      regName = regs[regAddr-0x180];
   else if ((regAddr >= 0x800) && (regAddr <= 0x80F)) // Register Misc
      regName = regs2[regAddr-0x800];
   else if (regAddr == 0xC04)
      regName = "FLASHBAR";
   else if (regAddr == 0xC05)
      regName = "RAMBAR";
   if (regName == NULL)
      regName = "unknown";

   return regName;
}

char const *getCFDebugRegName( unsigned int regAddr){
static const char *names[] = {"CSR","XCSR","CSR2","CSR3",NULL,"BAAR","AATR","TDR",
                             "PBR0","PBMR",NULL,NULL,"ABHR","ABLR","DBR","DBMR",
                             NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,
                             "PBR1","PBR2","PBR3",NULL,NULL,NULL,NULL,NULL,
                             };

const char *regName = NULL;

   if (regAddr <= sizeof(names)/sizeof(names[0]))
       regName = names[regAddr];

   if (regName == NULL)
      regName = "unknown";

   return regName;
}

char const *getCFRegName( unsigned int regAddr){
static const char *names[] = {"D0","D1","D2","D3","D4","D5","D6","D7",
                              "A0","A1","A2","A3","A4","A5","A6","USP"};

const char *regName = NULL;

   if (regAddr <= sizeof(names)/sizeof(names[0]))
       regName = names[regAddr];

   if (regName == NULL)
      regName = "unknown";

   return regName;
}



